@@ -31,6 +31,7 @@ from gajim.common.util.uri import get_file_path_from_uri
 from gajim.gtk.alert import InformationAlertDialog
 from gajim.gtk.filechoosers import AvatarFileChooserButton
 from gajim.gtk.util.classes import SignalManager
+from gajim.gtk.util.misc import has_min_gtk_version
 
 log = logging.getLogger("gajim.gtk.avatar_selector")
 
@@ -118,9 +119,9 @@ class AvatarSelector(Gtk.Box, SignalManager):
         self.prepare_crop_area(str(paths[0]))
 
     def _on_drop_accept(self, _target: Gtk.DropTarget, drop: Gdk.Drop) -> bool:
-        if app.is_display(Display.X11):
-            # DND on X11 freezes due to a GTK bug:
-            # https://dev.gajim.org/gajim/gajim/-/issues/12313
+        # DND on X11 freezes due to a GTK bug:
+        # https://dev.gajim.org/gajim/gajim/-/issues/12313
+        if app.is_display(Display.X11) and not has_min_gtk_version("4.20.1"):
             return False
 
         formats = drop.get_formats()
@@ -294,7 +295,6 @@ class CropArea(Gtk.DrawingArea, SignalManager):
         _width: int,
         _height: int,
     ) -> None:
-
         if self._browse_pixbuf is None:
             return
 
@@ -625,7 +625,6 @@ class CropArea(Gtk.DrawingArea, SignalManager):
             or self._pixbuf.get_width() != allocated_width
             or self._pixbuf.get_height() != allocated_height
         ):
-
             self._pixbuf = GdkPixbuf.Pixbuf.new(
                 GdkPixbuf.Colorspace.RGB,
                 self._browse_pixbuf.get_has_alpha(),
