@@ -47,6 +47,7 @@ from gajim.gtk.message_actions_box import MessageActionsBox
 from gajim.gtk.message_input import MessageInputTextView
 from gajim.gtk.util.classes import SignalManager
 from gajim.gtk.util.misc import allow_send_message
+from gajim.gtk.util.misc import has_min_gtk_version
 from gajim.gtk.util.window import open_window
 
 log = logging.getLogger("gajim.gtk.chatstack")
@@ -308,37 +309,31 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
     def _on_room_password_required(
         self, _contact: GroupchatContact, _signal_name: str
     ) -> None:
-
         self._show_chat_function_page(FunctionMode.PASSWORD_REQUEST)
 
     def _on_room_captcha_challenge(
         self, contact: GroupchatContact, _signal_name: str
     ) -> None:
-
         self._show_chat_function_page(FunctionMode.CAPTCHA_REQUEST)
 
     def _on_room_captcha_error(
         self, _contact: GroupchatContact, _signal_name: str, error: str
     ) -> None:
-
         self._show_chat_function_page(FunctionMode.CAPTCHA_ERROR, error)
 
     def _on_room_creation_failed(
         self, _contact: GroupchatContact, _signal_name: str, error: str
     ) -> None:
-
         self._show_chat_function_page(FunctionMode.CREATION_FAILED, error)
 
     def _on_room_join_failed(
         self, _contact: GroupchatContact, _signal_name: str, error: str
     ) -> None:
-
         self._show_chat_function_page(FunctionMode.JOIN_FAILED, error)
 
     def _on_room_config_failed(
         self, _contact: GroupchatContact, _signal_name: str, error: str
     ) -> None:
-
         self._show_chat_function_page(FunctionMode.CONFIG_FAILED)
 
     def _on_muc_state_changed(
@@ -353,7 +348,6 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
         _user_contact: GroupchatParticipant,
         _event: events.MUCUserJoined,
     ) -> None:
-
         self._update_group_chat_actions(contact)
 
     def _on_user_role_changed(
@@ -363,7 +357,6 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
         _user_contact: GroupchatParticipant,
         _event: events.MUCUserRoleChanged,
     ) -> None:
-
         self._update_group_chat_actions(contact)
 
     def _on_user_affiliation_changed(
@@ -387,7 +380,6 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
     def _on_account_state(
         self, event: events.AccountConnected | events.AccountDisconnected
     ) -> None:
-
         if self._current_contact is None:
             return
 
@@ -449,7 +441,6 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
         self._issue_notification(event.account, message)
 
     def _issue_notification(self, account: str, message: Message) -> None:
-
         text = message.text
         assert text is not None
 
@@ -581,7 +572,6 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
         pass
 
     def _on_action(self, action: Gio.SimpleAction, param: GLib.Variant | None) -> None:
-
         if self.get_visible_child_name() != "controls":
             return
 
@@ -745,7 +735,9 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
     def _on_drop_accept(self, _target: Gtk.DropTarget, _drop: Gdk.Drop) -> bool:
         # DND on X11 freezes due to a GTK bug:
         # https://dev.gajim.org/gajim/gajim/-/issues/12313
-        return not app.is_display(Display.X11)
+        if app.is_display(Display.X11):
+            return has_min_gtk_version("4.20.1")
+        return True
 
     def _on_file_drop(
         self,
@@ -754,7 +746,6 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
         _x: float,
         _y: float,
     ) -> bool:
-
         if value is None:
             log.debug("Drop received, but value is None")
             return False
@@ -821,7 +812,6 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
         data: str | None = None,
         files: list[str] | None = None,
     ) -> None:
-
         assert self._current_contact is not None
         self._chat_function_page.set_mode(
             self._current_contact, function_mode, data, files
@@ -844,9 +834,33 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
     def _on_function_message(
         self, _function_page: ChatFunctionPage, message: str
     ) -> None:
-
         self._chat_control.add_info_message(message)
 
+    def check_send_preconditons(self) -> bool:
+        contact = self._current_contact
+        assert contact is not None
+
+        client = app.get_client(contact.account)
+
+        encryption = contact.settings.get("encryption")
+        if not encryption:
+            return True
+
+        if encryption == "OMEMO":
+            return client.get_module("OMEMO").check_send_preconditions(contact)
+
+        if encryption not in app.plugin_manager.encryption_plugins:
+            InformationAlertDialog(
+                _("Encryption Error"), _("Missing necessary encryption plugin")
+            )
+            return False
+
+        self._chat_control.sendmessage = True
+        app.plugin_manager.extension_point(
+            "send_message" + encryption, self._chat_control
+        )
+        return self._chat_control.sendmessage
+
     def _on_send_message(self) -> None:
         message = self._message_action_box.get_text()
         if message.startswith("//"):
@@ -858,24 +872,8 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
 
         client = app.get_client(contact.account)
 
-        encryption = contact.settings.get("encryption")
-        if encryption == "OMEMO":
-            if not client.get_module("OMEMO").check_send_preconditions(contact):
-                return
-
-        elif encryption:
-            if encryption not in app.plugin_manager.encryption_plugins:
-                InformationAlertDialog(
-                    _("Encryption Error"), _("Missing necessary encryption plugin")
-                )
-                return
-
-            self._chat_control.sendmessage = True
-            app.plugin_manager.extension_point(
-                "send_message" + encryption, self._chat_control
-            )
-            if not self._chat_control.sendmessage:
-                return
+        if not self.check_send_preconditons():
+            return
 
         message = remove_invalid_xml_chars(message)
         if message in ("", "\n"):
@@ -929,7 +927,6 @@ class ChatStack(Gtk.Stack, EventHelper, SignalManager):
     def _on_command_signal(
         self, _chat_commands: ChatCommands, signal_name: str, text: str
     ) -> None:
-
         is_error = signal_name != "command-result"
         self._chat_control.add_command_output(text, is_error)
 
