@@ -18,6 +18,7 @@ from gi.repository import Gtk
 from nbxmpp.const import Affiliation
 
 from gajim.common import app
+from gajim.common import i18n
 from gajim.common import types
 from gajim.common.const import AvatarSize
 from gajim.common.events import MUCNicknameChanged
@@ -35,6 +36,7 @@ from gajim.gtk.structs import AddChatActionParams
 from gajim.gtk.tooltips import GCTooltip
 from gajim.gtk.util.classes import SignalManager
 from gajim.gtk.util.misc import get_ui_string
+from gajim.gtk.util.misc import scroll_to
 from gajim.gtk.widgets import GajimPopover
 
 log = logging.getLogger("gajim.gtk.groupchat_roster")
@@ -68,9 +70,12 @@ def get_group_from_contact(contact: types.GroupchatParticipant) -> tuple[str, st
 
 
 class GroupchatRoster(Gtk.Revealer, EventHelper):
+    __gtype_name__ = "GroupchatRoster"
+
     def __init__(self) -> None:
         Gtk.Revealer.__init__(self)
         EventHelper.__init__(self)
+        self.set_name("GroupchatRoster")
 
         self._contact = None
         self._scroll_id = None
@@ -94,6 +99,24 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
             lambda *args: self._ui.search_entry.set_text(""),
         )
 
+        self.bind_property(
+            "total-count",
+            self._ui.participants_count_label,
+            "label",
+            GObject.BindingFlags.SYNC_CREATE,
+            transform_to=self._transform_count_to_label,
+        )
+
+    @GObject.Property(type=int)
+    def total_count(self) -> int:
+        return self._contact_view.get_count()
+
+    @staticmethod
+    def _transform_count_to_label(binding: GObject.Binding, count: int) -> str:
+        return i18n.ngettext(
+            "%(count)s Participant", "%(count)s Participants", count
+        ) % {"count": count}
+
     def _hide_roster(self, hide_roster: bool, *args: Any) -> None:
         transition = Gtk.RevealerTransitionType.SLIDE_RIGHT
         if not hide_roster:
@@ -167,11 +190,16 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
             self._add_contact(participant)
 
         self._contact_view.bind_model()
+        self.notify("total-count")
+
+        # GTK Bug: ListView does not fully scroll to top when headers are used
+        GLib.idle_add(scroll_to, self._ui.scrolled, "top")
 
     def _unload_roster(self) -> None:
         log.info("Unload Roster")
         self._contact_view.unbind_model()
         self._contact_view.remove_all()
+        self.notify("total-count")
 
         assert self._contact is not None
         self._contact.multi_disconnect(self, CONTACT_SIGNALS)
@@ -181,7 +209,6 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         _list_view: Gtk.ListView,
         position: int,
     ) -> None:
-
         item = self._contact_view.get_listitem(position)
         assert item is not None
 
@@ -194,8 +221,11 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         assert disco is not None
 
         muc_prefer_direct_msg = app.settings.get("muc_prefer_direct_msg")
-        if disco.muc_is_nonanonymous and muc_prefer_direct_msg:
-            assert participant_contact.real_jid is not None
+        if (
+            disco.muc_is_nonanonymous
+            and muc_prefer_direct_msg
+            and participant_contact.real_jid is not None
+        ):
             dm_params = AddChatActionParams(
                 account=self._contact.account,
                 jid=participant_contact.real_jid,
@@ -229,12 +259,20 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
 
         self._scroll_id = GLib.timeout_add(100, _scroll_to)
 
-    def _add_contact(self, contact: types.GroupchatParticipant) -> None:
+    def _add_contact(
+        self, contact: types.GroupchatParticipant, notify: bool = False
+    ) -> None:
         item = GroupchatContactListItem(contact=contact)
         self._contact_view.add(item)
+        if notify:
+            self.notify("total-count")
 
-    def _remove_contact(self, contact: types.GroupchatParticipant) -> None:
+    def _remove_contact(
+        self, contact: types.GroupchatParticipant, notify: bool = False
+    ) -> None:
         self._contact_view.remove(contact)
+        if notify:
+            self.notify("total-count")
 
     def _on_contact_changed(
         self,
@@ -243,12 +281,11 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         user_contact: types.GroupchatParticipant,
         *args: Any,
     ) -> None:
-
         if signal_name == "user-joined":
-            self._add_contact(user_contact)
+            self._add_contact(user_contact, notify=True)
 
         elif signal_name == "user-left":
-            self._remove_contact(user_contact)
+            self._remove_contact(user_contact, notify=True)
 
         else:
             self._remove_contact(user_contact)
@@ -262,14 +299,12 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         old_contact: types.GroupchatParticipant,
         new_contact: types.GroupchatParticipant,
     ) -> None:
-
         self._remove_contact(old_contact)
         self._add_contact(new_contact)
 
     def _on_muc_state_changed(
         self, contact: GroupchatContact, _signal_name: str
     ) -> None:
-
         if contact.is_joined:
             self._load_roster()
 
@@ -283,7 +318,6 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
 class GroupchatContactListView(Gtk.ListView):
     def __init__(self) -> None:
         Gtk.ListView.__init__(self)
-        self.add_css_class("p-12")
 
         self._model = Gio.ListStore(item_type=GroupchatContactListItem)
 
@@ -332,6 +366,9 @@ class GroupchatContactListView(Gtk.ListView):
     def unbind_model(self) -> None:
         self._sort_model.set_model(None)
 
+    def get_count(self) -> int:
+        return self._model.get_n_items()
+
     @staticmethod
     def _on_factory_setup(
         _factory: Gtk.SignalListItemFactory, list_item: Gtk.ListItem, view_item: Any
@@ -363,7 +400,6 @@ class GroupchatContactListView(Gtk.ListView):
         obj2: Any,
         _user_data: object | None,
     ) -> int:
-
         group1_index = AffiliationRoleSortOrder[obj1.group]
         group2_index = AffiliationRoleSortOrder[obj2.group]
         if group1_index == group2_index:
@@ -376,7 +412,6 @@ class GroupchatContactListView(Gtk.ListView):
         obj2: Any,
         _user_data: object | None,
     ) -> int:
-
         if obj1.contact.is_self or obj2.contact.is_self:
             return -1 if obj1.contact.is_self else 1
 
@@ -438,7 +473,7 @@ class GroupchatContactListItem(GObject.Object):
         )
 
     def __repr__(self) -> str:
-        return f"GroupchatContactListItem: {self.props.nick}"
+        return f"GroupchatContactListItem: {self.nick}"
 
 
 @Gtk.Template(string=get_ui_string("groupchat_contact_view_item.ui"))
@@ -532,7 +567,6 @@ class GroupchatContactViewItem(Gtk.Grid, SignalManager):
         x: float,
         y: float,
     ) -> bool:
-
         assert self._contact is not None
         participant = self._contact
         if participant.is_self:
@@ -559,7 +593,6 @@ class GroupchatContactViewItem(Gtk.Grid, SignalManager):
         _keyboard_mode: bool,
         tooltip: Gtk.Tooltip,
     ) -> bool:
-
         assert self._contact is not None
         value, widget = self._tooltip.get_tooltip(self._contact)
         tooltip.set_custom(widget)
