@@ -10,9 +10,6 @@ from typing import Literal
 from typing import TYPE_CHECKING
 
 import logging
-import os
-import shutil
-from pathlib import Path
 
 from gi.repository import Adw
 from gi.repository import Gdk
@@ -38,9 +35,6 @@ from gajim.common.modules.contacts import GroupchatParticipant
 from gajim.common.modules.contacts import ResourceContact
 from gajim.common.storage.archive.const import MessageType
 from gajim.common.util.uri import InvalidUri
-from gajim.common.util.uri import open_file
-from gajim.common.util.uri import open_uri
-from gajim.common.util.uri import show_in_folder
 from gajim.common.util.uri import XmppIri
 
 from gajim.gtk.about import AboutDialog
@@ -78,6 +72,12 @@ if TYPE_CHECKING:
 if app.is_display(Display.X11):
     from gi.repository import GdkX11
 
+if app.is_display(Display.WIN32):
+    import gi
+
+    gi.require_version("GdkWin32", "4.0")
+    from gi.repository import GdkWin32
+
 log = logging.getLogger("gajim.gtk.main")
 
 
@@ -88,6 +88,7 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     _header_bar: Adw.HeaderBar = Gtk.Template.Child()
     _app_side_bar: AppSideBar = Gtk.Template.Child()
     _main_stack: MainStack = Gtk.Template.Child()
+    _toast_overlay: Adw.ToastOverlay = Gtk.Template.Child()
 
     def __init__(self) -> None:
         app.window = self
@@ -99,6 +100,18 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
         self.set_title(GLib.get_application_name())
         self.set_default_icon_name("gajim")
 
+        self._startup_finished: bool = False
+
+        self._emoji_chooser: EmojiChooser | None = None
+        self._about_dialog = None
+        self._chat_page = self._main_stack.get_chat_page()
+
+        self._prepare_window()
+
+    def init(self) -> None:
+        """Init is called at a later point, so that the (empty) window can be
+        displayed early, indicating a progressing Gajim startup.
+        """
         self._about_dialog = AboutDialog()
 
         self._add_actions()
@@ -106,11 +119,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
         self._add_stateful_actions()
         self._connect_actions()
 
-        self._startup_finished: bool = False
-
-        self._emoji_chooser: EmojiChooser | None = None
-
-        self._chat_page = self._main_stack.get_chat_page()
         self._app_side_bar.set_chat_page(self._chat_page)
 
         self.connect("notify::is-active", self._on_window_active)
@@ -153,8 +161,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
 
         app.ged.raise_event(events.RegisterActions())
 
-        self._prepare_window()
-
         chat_list_stack = self._chat_page.get_chat_list_stack()
         app.app.systray.connect_unread_widget(chat_list_stack, "unread-count-changed")
         chat_list_stack.connect("chat-selected", self._on_chat_selected)
@@ -167,6 +173,7 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
 
     @property
     def about_dialog(self) -> AboutDialog:
+        assert self._about_dialog is not None
         return self._about_dialog
 
     def get_action(self, name: str) -> Gio.SimpleAction:
@@ -192,6 +199,9 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
 
         return self._emoji_chooser
 
+    def show_toast(self, toast: Adw.Toast) -> None:
+        self._toast_overlay.add_toast(toast)
+
     def show(self) -> None:
         self.present()
 
@@ -202,14 +212,15 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
         toplevel.set_skip_taskbar_hint(value)
 
     def set_urgency_hint(self, value: bool) -> None:
-        if not app.is_display(Display.X11):
-            return
-
         if not app.settings.get("use_urgency_hint"):
             return
 
-        toplevel = cast(GdkX11.X11Surface, self.get_surface())
-        toplevel.set_urgency_hint(value)
+        if app.is_display(Display.X11):
+            toplevel = cast(GdkX11.X11Surface, self.get_surface())
+            toplevel.set_urgency_hint(value)
+        elif app.is_display(Display.WIN32):
+            toplevel = cast(GdkWin32.Win32Surface, self.get_surface())
+            toplevel.set_urgency_hint(value)
 
     def mark_workspace_as_read(self, workspace: str) -> None:
         chat_list_stack = self._chat_page.get_chat_list_stack()
@@ -221,7 +232,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     def _mark_workspace_as_read(
         self, _action: Gio.SimpleAction, param: GLib.Variant
     ) -> None:
-
         workspace_id = param.get_string() or None
         if workspace_id is not None:
             self.mark_workspace_as_read(workspace_id)
@@ -263,7 +273,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
         _keycode: int,
         state: Gdk.ModifierType,
     ) -> bool:
-
         if keyval == Gdk.KEY_space:
             self.get_chat_stack().get_message_input().grab_focus()
             return Gdk.EVENT_STOP
@@ -273,12 +282,10 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     def _on_client_state_changed(
         self, client: Client, _signal_name: str, state: SimpleClientState
     ) -> None:
-
         app.app.set_account_actions_state(client.account, state.is_connected)
         app.app.update_app_actions_state()
 
     def _on_client_resume_successful(self, client: Client, _signal_name: str) -> None:
-
         app.app.update_feature_actions_state(client.account)
 
     @staticmethod
@@ -293,7 +300,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
 
     @staticmethod
     def _on_plain_connection(event: events.PlainConnection) -> None:
-
         def _on_response(response_id: str) -> None:
             if response_id == "connect":
                 event.connect()
@@ -435,12 +441,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
             ("increase-app-font-size", self._on_app_font_size_action),
             ("decrease-app-font-size", self._on_app_font_size_action),
             ("reset-app-font-size", self._on_app_font_size_action),
-            ("preview-download", self._on_preview_action),
-            ("preview-open", self._on_preview_action),
-            ("preview-save-as", self._on_preview_action),
-            ("preview-open-folder", self._on_preview_action),
-            ("preview-copy-link", self._on_preview_action),
-            ("preview-open-link", self._on_preview_action),
             ("copy-message", self._on_copy_message),
             ("retract-message", self._on_retract_message),
             ("moderate-message", self._on_moderate_message),
@@ -474,7 +474,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     def _on_action(
         self, action: Gio.SimpleAction, _param: GLib.Variant | None
     ) -> int | None:
-
         action_name = action.get_name()
         log.info("Activate action: %s", action_name)
 
@@ -487,30 +486,37 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
 
         control = self.get_control()
         if control.has_active_chat():
+            contact = control.get_contact()
+            assert contact is not None
+
             if action_name == "change-nickname":
                 app.window.activate_action("win.muc-change-nickname", None)
                 return None
 
             if action_name == "change-subject":
-                open_window("GroupchatDetails", contact=control.contact, page="manage")
+                open_window("GroupchatDetails", contact=contact, page="manage")
                 return None
 
             if action_name == "escape":
                 if app.settings.get("escape_key_closes"):
-                    self._chat_page.remove_chat(
-                        control.contact.account, control.contact.jid
-                    )
-                    return None
+                    self._chat_page.remove_chat(contact.account, contact.jid)
+                else:
+                    workspace_id = self.get_active_workspace()
+                    if workspace_id is not None:
+                        app.window.activate_workspace(workspace_id)
+
+                return None
 
             elif action_name == "close-chat":
-                self._chat_page.remove_chat(
-                    control.contact.account, control.contact.jid
-                )
+                self._chat_page.remove_chat(contact.account, contact.jid)
                 return None
 
         if action_name == "escape" and app.settings.get("escape_key_closes"):
             self.close()
 
+        elif action_name == "close-chat":
+            self.close()
+
         elif action_name == "restore-chat":
             self._chat_page.restore_chat()
 
@@ -539,7 +545,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     def _on_app_font_size_action(
         self, action: Gio.SimpleAction, _param: GLib.Variant
     ) -> None:
-
         action_name = action.get_name()
         if action_name == "reset-app-font-size":
             app.settings.set_app_setting("app_font_size", None)
@@ -562,131 +567,7 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
         app.settings.set_app_setting("app_font_size", new_app_font_size)
         app.css_config.apply_app_font_size()
 
-    def _on_preview_action(self, action: Gio.SimpleAction, param: GLib.Variant) -> None:
-
-        action_name = action.get_name()
-        preview = app.preview_manager.get_preview(param.get_string())
-        if preview is None:
-            return
-
-        if action_name == "preview-download":
-            if not preview.orig_exists:
-                app.preview_manager.download_content(preview, force=True)
-
-        elif action_name == "preview-open":
-            if preview.is_geo_uri:
-                open_uri(preview.uri)
-                return
-
-            if not preview.orig_exists:
-                app.preview_manager.download_content(preview, force=True)
-                return
-
-            assert preview.orig_path is not None
-            open_file(preview.orig_path)
-
-        elif action_name == "preview-save-as":
-
-            def _on_save_finished(
-                file_dialog: Gtk.FileDialog, result: Gio.AsyncResult
-            ) -> None:
-                try:
-                    gfile = file_dialog.save_finish(result)
-                except GLib.Error as e:
-                    if e.code == 2:
-                        # User dismissed dialog, do nothing
-                        return
-
-                    log.exception(e)
-                    InformationAlertDialog(
-                        _("Could Not Save File"),
-                        _("Could not save file to selected directory."),
-                        parent=self,
-                    )
-                    return
-
-                assert preview is not None
-                assert preview.orig_path is not None
-
-                path = gfile.get_path()
-                assert path is not None
-                target_path = Path(path)
-                orig_ext = preview.orig_path.suffix
-                new_ext = target_path.suffix
-                if orig_ext != new_ext:
-                    # Windows file chooser selects the full file name including
-                    # extension. Starting to type will overwrite the extension
-                    # as well. Restore the extension if it's lost.
-                    target_path = target_path.with_suffix(orig_ext)
-                dirname = target_path.parent
-                if not os.access(dirname, os.W_OK):
-                    InformationAlertDialog(
-                        _("Directory Not Writable"),
-                        _(
-                            'Directory "%s" is not writable. '
-                            "You do not have the proper permissions to "
-                            "create files in this directory."
-                        )
-                        % dirname,
-                        parent=self,
-                    )
-                    return
-
-                try:
-                    shutil.copyfile(preview.orig_path, target_path)
-                except PermissionError as e:
-                    InformationAlertDialog(
-                        _("Could Not Save File"),
-                        _(
-                            "You do not have permissions for this directory.\n"
-                            "Error: %s."
-                        )
-                        % e,
-                        parent=self,
-                    )
-                    return
-
-                app.settings.set("last_save_dir", str(target_path.parent))
-
-            if not preview.orig_exists:
-                app.preview_manager.download_content(preview, force=True)
-                return
-
-            gfile = None
-            last_dir = app.settings.get("last_save_dir")
-            if last_dir:
-                gfile = Gio.File.new_for_path(last_dir)
-
-            dialog = Gtk.FileDialog(
-                initial_folder=gfile,
-                initial_name=preview.filename,
-            )
-            dialog.save(self, None, _on_save_finished)
-
-        elif action_name == "preview-open-folder":
-            if not preview.orig_exists:
-                app.preview_manager.download_content(preview, force=True)
-                return
-
-            assert preview.orig_path is not None
-            show_in_folder(preview.orig_path)
-
-        elif action_name == "preview-copy-link":
-            self.get_clipboard().set(preview.uri)
-
-        elif action_name == "preview-open-link":
-            if preview.is_aes_encrypted:
-                if preview.is_geo_uri:
-                    open_uri(preview.uri)
-                    return
-
-                assert preview.orig_path
-                open_file(preview.orig_path)
-            else:
-                open_uri(preview.uri)
-
     def _on_copy_message(self, _action: Gio.SimpleAction, param: GLib.Variant) -> None:
-
         self.get_clipboard().set(param.get_string())
 
     @actionmethod
@@ -704,7 +585,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     def _on_muc_user_block(
         self, _action: Gio.SimpleAction, params: OccupantParam
     ) -> None:
-
         def _on_response() -> None:
             client = app.get_client(params.account)
             client.get_module("MucBlocking").set_block_occupants(
@@ -736,7 +616,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     def _on_muc_user_unblock(
         self, _action: Gio.SimpleAction, params: OccupantParam
     ) -> None:
-
         def _on_response() -> None:
             client = app.get_client(params.account)
             client.get_module("MucBlocking").set_block_occupants(
@@ -754,7 +633,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     def _on_retract_message(
         self, _action: Gio.SimpleAction, params: RetractMessageParam
     ) -> None:
-
         def _on_response() -> None:
             client = app.get_client(params.account)
             contact = client.get_module("Contacts").get_contact(params.jid)
@@ -777,7 +655,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     def _on_moderate_message(
         self, _action: Gio.SimpleAction, params: ModerateMessageParam
     ) -> None:
-
         def _on_response(reason: str) -> None:
             client = app.get_client(params.account)
             client.get_module("MUC").moderate_messages(
@@ -839,7 +716,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     def _on_delete_message_locally(
         self, _action: Gio.SimpleAction, params: DeleteMessageParam
     ) -> None:
-
         def _on_response() -> None:
             app.storage.archive.delete_message(params.pk)
             app.ged.raise_event(
@@ -910,11 +786,13 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
             return
 
         if self.is_active():
-            client = app.get_client(control.contact.account)
+            contact = control.get_contact()
+            assert contact is not None
+            client = app.get_client(contact.account)
             chat_stack = self._chat_page.get_chat_stack()
             msg_action_box = chat_stack.get_message_action_box()
             client.get_module("Chatstate").set_mouse_activity(
-                control.contact, msg_action_box.msg_textview.has_text
+                contact, msg_action_box.msg_textview.has_text
             )
 
     def _on_chat_selected(self, *args: Any) -> None:
@@ -989,7 +867,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
         self._app_side_bar.highlight_dnd_targets(highlight)
 
     def _add_workspace(self, _action: Gio.SimpleAction, param: GLib.Variant) -> None:
-
         workspace_id = param.get_string()
         if workspace_id:
             self.add_workspace(workspace_id)
@@ -997,7 +874,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     def add_workspace(
         self, workspace_id: str | None = None, switch: bool = True
     ) -> str:
-
         if workspace_id is None:
             workspace_id = app.settings.add_workspace(_("My Workspace"))
 
@@ -1017,7 +893,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
         open_window("WorkspaceDialog", workspace_id=workspace_id)
 
     def _remove_workspace(self, _action: Gio.SimpleAction, param: GLib.Variant) -> None:
-
         workspace_id = param.get_string() or None
         if workspace_id is None:
             workspace_id = self.get_active_workspace()
@@ -1075,7 +950,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     def _activate_workspace(
         self, _action: Gio.SimpleAction, param: GLib.Variant
     ) -> None:
-
         workspace_id = param.get_string()
         if workspace_id:
             self.activate_workspace(workspace_id)
@@ -1097,7 +971,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     def _get_suitable_workspace(
         self, account: str, jid: JID, private_chat: bool = False
     ) -> str:
-
         if private_chat:
             # Try to add private chat to the same workspace the MUC resides in
             chat_list_stack = self._chat_page.get_chat_list_stack()
@@ -1116,19 +989,16 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
         return self._app_side_bar.get_first_workspace()
 
     def _add_group_chat(self, _action: Gio.SimpleAction, param: GLib.Variant) -> None:
-
         account, jid, select = param.unpack()
         self.add_group_chat(account, JID.from_string(jid), select)
 
     def add_group_chat(self, account: str, jid: JID, select: bool = False) -> None:
-
         workspace_id = self._get_suitable_workspace(account, jid)
         self._chat_page.add_chat_for_workspace(
             workspace_id, account, jid, "groupchat", select=select
         )
 
     def _add_chat(self, _action: Gio.SimpleAction, param: GLib.Variant) -> None:
-
         params = AddChatActionParams.from_variant(param)
         self.add_chat(params.account, params.jid, params.type, params.select)
 
@@ -1140,14 +1010,12 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
         select: bool = False,
         message: str | None = None,
     ) -> None:
-
         workspace_id = self._get_suitable_workspace(account, jid)
         self._chat_page.add_chat_for_workspace(
             workspace_id, account, jid, type_, select=select, message=message
         )
 
     def add_private_chat(self, account: str, jid: JID, select: bool = False) -> None:
-
         workspace_id = self._get_suitable_workspace(account, jid, private_chat=True)
         self._chat_page.add_chat_for_workspace(
             workspace_id, account, jid, "pm", select=select
@@ -1191,7 +1059,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     def is_message_correctable(
         self, contact: types.ChatContactT, message_id: str
     ) -> bool:
-
         chat_stack = self._chat_page.get_chat_stack()
         last_message_id = chat_stack.get_last_message_id(contact)
         if last_message_id is None or last_message_id != message_id:
@@ -1220,7 +1087,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
         *,
         is_sync: bool = False,
     ) -> None:
-
         unread_count = self.get_chat_unread_count(account, jid, include_silent=True)
 
         self.set_urgency_hint(False)
@@ -1269,11 +1135,13 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
         if not control.has_active_chat():
             return
 
+        contact = control.get_contact()
+        assert contact is not None
+
         if control.view_is_at_bottom():
-            self.mark_as_read(control.contact.account, control.contact.jid)
+            self.mark_as_read(contact.account, contact.jid)
 
     def get_preferred_ft_method(self, contact: types.ChatContactT) -> str | None:
-
         httpupload_enabled = app.window.get_action_enabled("send-file-httpupload")
         return "httpupload" if httpupload_enabled else None
         # TODO Jingle FT
@@ -1306,7 +1174,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
         nickname: str | None = None,
         password: str | None = None,
     ) -> None:
-
         jid_ = JID.from_string(jid)
         if not self.chat_exists(account, jid_):
             client = app.get_client(account)
@@ -1317,7 +1184,6 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     def start_chat_from_jid(
         self, account: str, jid: str, message: str | None = None
     ) -> None:
-
         jid_ = JID.from_string(jid)
         if self.chat_exists(account, jid_):
             self.select_chat(account, jid_)
@@ -1403,7 +1269,20 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
 
         self._set_startup_finished()
 
+    def _is_history_sync(self, event: events.MessageReceived) -> bool:
+        if event.mam is None:
+            return False
+
+        win = get_app_window("HistorySyncAssistant", account=event.account)
+        if win is None:
+            return False
+
+        return win.get_active_query_id() == event.mam.query_id
+
     def _on_message_received(self, event: events.MessageReceived) -> bool | None:
+        if self._is_history_sync(event):
+            return ged.STOP_PROPAGATION
+
         if self.chat_exists(event.account, event.jid):
             return
 
@@ -1457,6 +1336,8 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
             self.add_chat(event.account, event.jid, "chat")
 
     def quit(self) -> None:
+        self.show_toast(Adw.Toast(title=_("Gajim is quitting…"), timeout=0))
+
         if self.is_visible():
             window_width, window_height = self.get_width(), self.get_height()
             app.settings.set("mainwin_width", window_width)
