@@ -87,13 +87,13 @@ def parse(path: Path, file: TextIOWrapper) -> str:
             continue
         klass = node.attrib["class"]
         if klass.startswith("GtkSource"):
-            klass = f'GtkSource.{klass.removeprefix("GtkSource")}'
+            klass = f"GtkSource.{klass.removeprefix('GtkSource')}"
         elif klass.startswith("Adw"):
-            klass = f'Adw.{klass.removeprefix("Adw")}'
+            klass = f"Adw.{klass.removeprefix('Adw')}"
         elif klass.startswith("Atk"):
-            klass = f'Atk.{klass.removeprefix("Atk")}'
+            klass = f"Atk.{klass.removeprefix('Atk')}"
         else:
-            klass = f'Gtk.{klass.removeprefix("Gtk")}'
+            klass = f"Gtk.{klass.removeprefix('Gtk')}"
 
         lines.append(ATTR % (id_.replace("-", "_"), klass))
 
@@ -103,8 +103,7 @@ def parse(path: Path, file: TextIOWrapper) -> str:
     if not lines:
         file.write("\n    pass")
     else:
-        for line in lines:
-            file.write(line)
+        file.writelines(lines)
     file.write("\n\n")
     return klass_name
 
